@@ -2,6 +2,8 @@
 Connect to `Telegram <https://telegram.org>`_ as a bot.
 
 Requirements:
+    Extra name: ``telegram``
+
     `aiohttp <https://aiohttp.readthedocs.io/en/latest/>`_
 
     `telethon <https://telethon.readthedocs.io/en/latest/>`_:
@@ -89,7 +91,8 @@ class _Schema:
 
     file = {"file_id": str,
             immp.Optional("file_name"): immp.Nullable(str),
-            immp.Optional("file_path"): immp.Nullable(str)}
+            immp.Optional("file_path"): immp.Nullable(str),
+            immp.Optional("mime_type"): immp.Nullable(str)}
 
     _sticker = {immp.Optional("emoji"): immp.Nullable(str), **file}
 
@@ -177,10 +180,13 @@ class _HiddenSender:
     # @GroupAnonymousBot, "Group": author of group chat messages on behalf of anonymous admins.
     anonymous_user_id = 1087968824
 
+    # @ChatsImportBot, "Imported Message": author of group chat messages imported from other apps.
+    import_user_id = 1474613229
+
     @classmethod
     def has(cls, value):
-        return value in (cls.hidden_channel_id, cls.hidden_chat_id,
-                         cls.service_user_id, cls.anonymous_user_id)
+        return value in (cls.hidden_channel_id, cls.hidden_chat_id, cls.service_user_id,
+                         cls.anonymous_user_id, cls.import_user_id)
 
 
 class TelegramUser(immp.User):
@@ -664,14 +670,18 @@ class TelegramMessage(immp.Message):
                 if message[key]:
                     obj = message[key]
                     break
-            if key == "animation":
+            mime = obj["mime_type"] or ""
+            if key == "animation" or mime.startswith("image/"):
                 type_ = immp.File.Type.image
-            elif key in ("video", "video_note"):
+            elif key in ("video", "video_note") or mime.startswith("video/"):
                 type_ = immp.File.Type.video
             else:
                 type_ = immp.File.Type.unknown
             attachments.append(await TelegramFile.from_id(telegram, obj["file_id"], type_,
                                                           obj["file_name"]))
+            if message["caption"]:
+                text = await TelegramRichText.from_bot_entities(telegram, message["caption"],
+                                                                message["caption_entities"])
         elif message["venue"]:
             attachments.append(immp.Location(latitude=message["venue"]["location"]["latitude"],
                                              longitude=message["venue"]["location"]["longitude"],
@@ -688,21 +698,33 @@ class TelegramMessage(immp.Message):
         elif not text:
             # No support for this message type.
             raise NotImplementedError
+        ignore_forward = False
+        if message["from"] and message["from"]["id"] == _HiddenSender.import_user_id:
+            # Import bot forwarded the message.  Copy the forward metadata to the main message.
+            ignore_forward = True
+            if message["forward_date"]:
+                at = datetime.fromtimestamp(message["forward_date"], timezone.utc)
+            if message["forward_from"]:
+                user = TelegramUser.from_bot_user(telegram, message["forward_from"])
+            elif message["forward_sender_name"]:
+                user = immp.User(real_name=message["forward_sender_name"])
         common = dict(id_=id_,
                       revision=revision,
                       at=at,
                       channel=channel,
                       user=user,
                       raw=message)
-        if message["forward_date"]:
+        if message["forward_date"] and not ignore_forward:
             # Event is a message containing another message.  Forwarded messages have no ID, so we
             # use a Message instead of a SentMessage here, unless they come from a channel.
-            forward_id = forward_channel = forward_user = None
+            forward_id = forward_channel = forward_date = forward_user = None
             chat = message["forward_from_chat"]
             sender = message["forward_from"]
             if chat and message["forward_from_message_id"]:
-                forward_channel = immp.Channel(telegram, chat["id"])
                 forward_id = "{}:{}".format(chat["id"], message["forward_from_message_id"])
+                forward_channel = immp.Channel(telegram, chat["id"])
+            if message["forward_date"]:
+                at = datetime.fromtimestamp(message["forward_date"], timezone.utc)
             if sender and not _HiddenSender.has(sender["id"]):
                 forward_user = TelegramUser.from_bot_user(telegram, sender)
             elif chat and not _HiddenSender.has(chat["id"]):
@@ -723,6 +745,7 @@ class TelegramMessage(immp.Message):
             if forward_id:
                 forward = immp.SentMessage(id_=forward_id,
                                            channel=forward_channel,
+                                           at=forward_date,
                                            **forward_common)
             else:
                 forward = immp.Message(**forward_common)
@@ -776,6 +799,7 @@ class TelegramMessage(immp.Message):
                 Parsed message object.
         """
         id_ = "{}:{}".format(message.chat_id, message.id)
+        at = message.date
         channel = immp.Channel(telegram, message.chat_id)
         edited = bool(message.edit_date)
         if edited:
@@ -815,7 +839,9 @@ class TelegramMessage(immp.Message):
             name = None
             sticker = False
             for attr in message.document.attributes:
-                if isinstance(attr, tl.types.DocumentAttributeSticker):
+                if isinstance(attr, tl.types.DocumentAttributeFilename):
+                    name = attr.file_name
+                elif isinstance(attr, tl.types.DocumentAttributeSticker):
                     type_ = immp.File.Type.image
                     if attr.alt and not text:
                         text = "sent {} sticker".format(attr.alt)
@@ -825,8 +851,12 @@ class TelegramMessage(immp.Message):
                     type_ = immp.File.Type.image
                 elif isinstance(attr, tl.types.DocumentAttributeVideo):
                     type_ = immp.File.Type.video
-                if isinstance(attr, tl.types.DocumentAttributeFilename):
-                    name = attr.file_name
+            if type_ == immp.File.Type.unknown:
+                mime = message.document.mime_type or ""
+                if mime.startswith("image/"):
+                    type_ = immp.File.Type.image
+                elif mime.startswith("video/"):
+                    type_ = immp.File.Type.video
             if telegram.config["stickers"] or not sticker:
                 try:
                     attach = await TelegramFile.from_id(telegram,
@@ -883,20 +913,33 @@ class TelegramMessage(immp.Message):
         if not text and not attachments:
             # No support for this message type.
             raise NotImplementedError
+        ignore_forward = False
+        if message.sender_id == _HiddenSender.import_user_id:
+            # Import bot forwarded the message.  Copy the forward metadata to the main message.
+            ignore_forward = True
+            if message.forward.date:
+                at = message.forward.date
+            if message.forward.sender_id:
+                sender = await message.forward.get_sender()
+                user = TelegramUser.from_proto_user(telegram, sender)
+            elif message.forward.from_name:
+                user = immp.User(real_name=message.forward.from_name)
         common = dict(id_=id_,
                       revision=revision,
-                      at=message.date,
+                      at=at,
                       channel=channel,
                       user=user,
                       raw=message)
-        if message.forward:
+        if message.forward and not ignore_forward:
             # Event is a message containing another message.  Forwarded messages have no ID, so we
             # use a Message instead of a SentMessage here, unless they come from a channel.
-            forward_id = forward_channel = forward_user = None
+            forward_id = forward_channel = forward_date = forward_user = None
             if message.forward.chat_id and message.forward.channel_post:
-                forward_channel = immp.Channel(telegram, message.forward.chat_id)
                 forward_id = "{}:{}".format(message.forward.chat_id,
                                             message.forward.channel_post)
+                forward_channel = immp.Channel(telegram, message.forward.chat_id)
+            if message.forward.date:
+                forward_date = message.forward.date
             if message.forward.sender_id and not _HiddenSender.has(message.forward.sender_id):
                 sender = await message.forward.get_sender()
                 forward_user = TelegramUser.from_proto_user(telegram, sender)
@@ -919,6 +962,7 @@ class TelegramMessage(immp.Message):
             if forward_id:
                 forward = immp.SentMessage(id_=forward_id,
                                            channel=forward_channel,
+                                           at=forward_date,
                                            **forward_common)
             else:
                 forward = immp.Message(**forward_common)
@@ -1302,9 +1346,11 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         if not self._client:
             log.debug("Client auth required to retrieve messages")
             return None
-        message = await self._client.get_messages(int(receipt.channel.source), ids=int(receipt.id))
+        id_ = int(receipt.id.split(":", 1)[1])
+        chat = int(receipt.channel.source)
+        message = await self._client.get_messages(chat, ids=id_)
         if not message:
-            log.debug("Failed to find message %d in %d", receipt.id, receipt.channel.source)
+            log.debug("Failed to find message %d in chat %d", id_, chat)
             return None
         try:
             return await TelegramMessage.from_proto_message(self, message)
@@ -1364,7 +1410,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
         quote = False
         rich = None
         if isinstance(msg.reply_to, immp.Receipt):
-            reply_to = int(msg.reply_to.id.split(":")[1])
+            reply_to = int(msg.reply_to.id.split(":", 1)[1])
         elif isinstance(msg.reply_to, immp.Message):
             quote = True
         if msg.text or quote:
@@ -1427,7 +1473,7 @@ class TelegramPlug(immp.HTTPOpenable, immp.Plug):
             # Generate requests for attached messages first.
             if isinstance(attach, immp.Receipt):
                 # Forward the messages natively using the given chat/ID.
-                forward_chat, forward_id = map(int, attach.id.split(":"))
+                forward_chat, forward_id = map(int, attach.id.split(":", 1))
                 requests.append(self._api("forwardMessage", _Schema.message,
                                           params={"chat_id": chat,
                                                   "from_chat_id": forward_chat,
